#include "Adafruit_VL53L0X.h"
#include "Servo.h"

#define LOX1_ADDRESS 0x30
#define LOX2_ADDRESS 0x31
#define LOX3_ADDRESS 0x32

#define SHT_LOX1 52
#define SHT_LOX2 53
#define SHT_LOX3 51

VL53L0X_RangingMeasurementData_t measure1;
VL53L0X_RangingMeasurementData_t measure2;
VL53L0X_RangingMeasurementData_t measure3;


Servo servo;
Servo motor;

Adafruit_VL53L0X lox1 = Adafruit_VL53L0X();
Adafruit_VL53L0X lox2 = Adafruit_VL53L0X();
Adafruit_VL53L0X lox3 = Adafruit_VL53L0X();


int p; 
int pd;
int d, error, last_error;
float kp = 1.0;
float kd = 7.0;
unsigned long myTime;

void setID() {
  // all reset
  digitalWrite(SHT_LOX1, LOW);    
  digitalWrite(SHT_LOX2, LOW);
  digitalWrite(SHT_LOX3, LOW);
  delay(10);
  // all unreset
  digitalWrite(SHT_LOX1, HIGH);
  digitalWrite(SHT_LOX2, HIGH);
  digitalWrite(SHT_LOX3, HIGH);
  delay(10);

  // activating LOX1 and resetting LOX2
  digitalWrite(SHT_LOX1, HIGH);
  digitalWrite(SHT_LOX2, LOW);
  digitalWrite(SHT_LOX3, LOW);

  // initing LOX1
  if(!lox1.begin(LOX1_ADDRESS)) {
    Serial.println(F("Failed to boot first VL53L0X"));
    while(1);
  }
  delay(10);

  // activating LOX2
  digitalWrite(SHT_LOX2, HIGH);
  delay(10);

  //initing LOX2
  if(!lox2.begin(LOX2_ADDRESS)) {
    Serial.println(F("Failed to boot second VL53L0X"));
    while(1);
  }

  // activating LOX3
  digitalWrite(SHT_LOX3, HIGH);
  delay(10);

  //initing LOX2
  if(!lox3.begin(LOX3_ADDRESS)) {
    Serial.println(F("Failed to boot third VL53L0X"));
    while(1);
  }
}

void read_three_sensors() {
  
  lox1.rangingTest(&measure1, false); // pass in 'true' to get debug data printout!
  lox2.rangingTest(&measure2, false); // pass in 'true' to get debug data printout!
  lox3.rangingTest(&measure3, false); // pass in 'true' to get debug data printout!

  Serial.print(F("Time: "));
  Serial.print(millis());
  Serial.print(F(" => "));
  
  
  // print sensor one reading
  Serial.print(F("1: "));
  if(measure1.RangeStatus != 4) {     // if not out of range
    Serial.print(measure1.RangeMilliMeter);
  } else {
    Serial.print(F("Out of range"));
  }
  
  Serial.print(F(" "));

  // print sensor two reading
  Serial.print(F("2: "));
  if(measure2.RangeStatus != 4) {
    Serial.print(measure2.RangeMilliMeter);
  } else {
    Serial.print(F("Out of range"));
  }
  
  Serial.print(F(" "));

  // print sensor three reading
  Serial.print(F("3: "));
  if(measure3.RangeStatus != 4) {
    Serial.print(measure3.RangeMilliMeter);
  } else {
    Serial.print(F("Out of range"));
  }

  Serial.println();
}


void setup() {
  Serial.begin(115200);

  // lox

  while (! Serial) { delay(1); }

  pinMode(SHT_LOX1, OUTPUT);
  pinMode(SHT_LOX2, OUTPUT);
  pinMode(SHT_LOX3, OUTPUT);

  Serial.println(F("Shutdown pins inited..."));

  digitalWrite(SHT_LOX1, LOW);
  digitalWrite(SHT_LOX2, LOW);
  digitalWrite(SHT_LOX3, LOW);

  Serial.println(F("All in reset mode...(pins are low)"));
  
  Serial.println(F("Starting..."));
  
  setID();

  lox1.startRangeContinuous();
  lox2.startRangeContinuous();
  lox3.startRangeContinuous();

  // servo and motor
  servo.attach(3);
  motor.attach(2);
  motor.writeMicroseconds(1500);
  servo.write(90);
  delay(5000);
}

void loop() {
//  read_three_sensors();
  myTime = millis();
  if (myTime <= 14700){
    if (lox1.isRangeComplete()) {
      motor.writeMicroseconds(1589);
      int distance_lox3 = lox3.readRange() / 10; // right
      int distance_lox1 = lox1.readRange() / 10; // forward
      int distance_lox2 = lox2.readRange() / 10; // left
      
      if (distance_lox3 > 100) distance_lox3 = 100;
      if (distance_lox1 > 100) distance_lox1 = 100;
      if (distance_lox2 > 100) distance_lox2 = 100;
      
      error = distance_lox1-distance_lox2;
      p = error * kp;
      d = (error - last_error) * kd;
      pd = p + d;
      int angle = pd + 65;
      if (angle > 135) angle = 135;
      if (angle < 35) angle = 35;
      servo.write(angle);
      Serial.println(angle);
      last_error = error;

      if (distance_lox3 < 20) {
        motor.writeMicroseconds(1500);
        delay(5000);
      }
      
  
  //    servo.writeMicroseconds(angle);
      
    }
  }
  else {
    motor.writeMicroseconds(1500);
    delay(5000);
  }
}





    
  

